require_dependency 'saaz/application_controller'

class Saaz::CustomersController < ApplicationController
  respond_to :json, :html

  def new
    set_resource_for_new
  end

  def create
    set_resource_for_create
    @resource.set_customers_on_subscription
    save_and_respond(@resource)
    after_create
  end

  def edit
    set_resource_for_edit
  end

  def update
    set_resource_for_update
    @resource.cards.create(card: params[:stripeToken])
    @resource.assign_attributes(resource_params)
    save_and_respond(@resource)
    after_update
  end

  protected

  def set_resource_for_new
    @resource = Saaz.config.customer_resource_class.new
  end

  def set_resource_for_create
    @resource = Saaz.config.customer_resource_class.new(resource_params)
  end

  def set_resource_for_edit
    set_resource_for_update
  end

  def set_resource_for_update
    @resource = current_user.customer
  end

  def after_create
    current_user.customer = @resource
    current_user.save!
  end

  def after_update; end

  def save_and_respond(resource)
    unless resource.save
      flash[:danger] = resource.errors.full_messages.to_sentence
    end

    respond_with resource do |fmt|
      fmt.html { redirect_to :back }
      fmt.json
    end
  end

  def resource_param_name
    Saaz.config.customer_resource_class.name.underscore.to_sym
  end

  def resource_params
    params.require(resource_param_name).
    merge(stripe_token: params[:stripeToken]).
    permit(:stripe_token, subscriptions_attributes: [ :plan_id ])
  end
end
