class Saaz::Customer < ActiveRecord::Base
  include Saaz::StripeObject

  attr_accessor :stripe_token, :stripe_description

  has_many :subscriptions, class_name: 'Saaz::Subscription'
  accepts_nested_attributes_for :subscriptions

  before_validation :create_stripe

  validates_associated :subscriptions
  validates :stripe_id, presence: true

  def to_stripe
    @stripe_instance ||= to_stripe_uncached
  end

  def to_stripe_uncached
    retval = Stripe::Customer.retrieve(stripe_id)
    delegate_all_to_stripe(retval)
    retval
  end

  def set_customers_on_subscription
    subscriptions.each do |sub|
      sub.customer = self.becomes(Saaz::Customer)
    end
  end

  def default_card_string
    card = self.cards.retrieve(self.default_card)
    "#{card.brand} ending in #{card.last4}"
  end

  protected

  def create_stripe
    return if stripe_id.present?

    opts = {}
    opts[:description] = stripe_description unless stripe_description.nil?
    opts[:card] = stripe_token unless stripe_token.nil?

    cust_obj = Stripe::Customer.create opts
    self.stripe_id = cust_obj.id
  end
end
