class Saaz::Subscription < ActiveRecord::Base
  include Saaz::StripeObject

  attr_accessor :plan_id

  belongs_to :customer, class_name: 'Saaz::Customer'

  before_validation :create_stripe

  validate :stripe_id, presence: true
  validate :customer, presence: true

  def to_stripe
    @stripe_instance ||= to_stripe_uncached
  end

  def to_stripe_uncached
    retval = customer.to_stripe.subscriptions.retrieve(stripe_id)
    delegate_all_to_stripe(retval)
    retval
  end

  def plan_id
    retval = @plan_id
    retval ||= to_stripe.plan.id rescue nil
    retval
  end

  def cancel_stripe
    to_stripe.delete(at_period_end: true)
  end

  protected

  def create_stripe
    return if stripe_id.present?

    customer.create_stripe
    sub_obj = customer.to_stripe.subscriptions.create(plan: plan_id)
    self.stripe_id = sub_obj.id
  end
end
