module Saaz::StripeObject
  extend ActiveSupport::Concern

  included do
    validate :update_stripe, if: :stripe_dirty
    attr_writer :stripe_dirty
  end

  def stripe_dirty
    @stripe_dirty || false
  end

  def update_stripe
    begin
      to_stripe.save
      self.stripe_dirty = false
    rescue Stripe::StripeError => e
      self.errors[:base] << e.message
    end
  end

  def delegate_to_stripe(name, &block)
    (class << self; self; end).class_eval do
      define_method name, &block
    end
  end

  def delegate_getter(attribute)
    return if self.respond_to?(attribute)
    delegate_to_stripe(attribute) do
      to_stripe.send(attribute)
    end
  end

  def delegate_setter(attribute)
    setter_name = (attribute.to_s + '=').to_sym
    return if self.respond_to?(setter_name)

    delegate_to_stripe(setter_name) do |v|
      self.stripe_dirty = true
      to_stripe.send(setter_name, v)
    end
  end

  def delegate_all_to_stripe(stripe_obj)
    stripe_obj.keys.each do |attribute|
      delegate_getter(attribute)
      delegate_setter(attribute)
    end
  end

  def method_missing(method_name, *args, &block)
    begin
      # Try the rails magic first
      super
    rescue NoMethodError => e
      # See if the wrapped stripe object defines this
      to_stripe
      if methods.include? method_name
        send(method_name, *args, &block)
      else
        fail e
      end
    end
  end
end
